@@ -19,12 +19,26 @@
 #ifndef _GOBBY_COMMON_HPP_
 #define _GOBBY_COMMON_HPP_
 
-#include <libintl.h>
-#include <locale.h>
+#include "net6/gettext_package.hpp"
+
+namespace Gobby
+{
+
+/** Initalises gettext for usage with Gobby. Translations a looked up in the
+ * given catalog.
+ */
+void init_gettext(net6::gettext_package& package);
+
+/** Translates a message from the gobby catalog.
+ */
+const char* _(const char* msgid);
+
+/** Translate pluralised message from the gobby catalog.
+ */
+const char* ngettext(const char* msgid,
+                     const char* msgid_plural,
+                     unsigned long int n);
 
-template<typename char_type>
-inline const char_type* _(const char_type* text) {
-	return gettext(text);
 }
 
 #endif // _GOBBY_COMMON_HPP_
