@@ -19,17 +19,27 @@
 #ifndef _GOBBY_ICON_HPP_
 #define _GOBBY_ICON_HPP_
 
-#include <glib/gtypes.h>
+#include <gtkmm/stockid.h>
+#include <gdkmm/pixbuf.h>
+#include <gtkmm/iconset.h>
+#include <gtkmm/iconfactory.h>
 
 namespace Gobby
 {
-	namespace Icon
+	class IconManager
 	{
-		/** The Gobby Icon as inline xpm data.
-		 */
-		extern const guint8 gobby[];
-	}
+	public:
+		static Gtk::StockID STOCK_USERLIST;
+
+		IconManager();
+
+		const Glib::RefPtr<Gdk::Pixbuf> gobby;
+		const Glib::RefPtr<Gdk::Pixbuf> userlist;
+
+	protected:
+		Gtk::IconSet m_is_userlist;
+		Glib::RefPtr<Gtk::IconFactory> m_icon_factory;
+	};
 }
 
 #endif // _GOBBY_ICON_HPP_
-
