@@ -22,16 +22,21 @@
 #include <gtkmm/window.h>
 #include <gtkmm/paned.h>
 #include <gtkmm/frame.h>
-#include <obby/error.hpp>
+#include <gtkmm/messagedialog.h>
 #include <obby/local_buffer.hpp>
+#include "features.hpp"
+#include "icon.hpp"
 #include "config.hpp"
 #include "header.hpp"
 #include "docwindow.hpp"
-#include "folder.hpp"
 #include "userlist.hpp"
+#include "documentlist.hpp"
+#include "finddialog.hpp"
+#include "gotodialog.hpp"
+#include "folder.hpp"
 #include "chat.hpp"
 #include "statusbar.hpp"
-#include "features.hpp"
+#include "dragdrop.hpp"
 #ifdef WITH_HOWL
 #include <obby/zeroconf.hpp>
 #endif
@@ -42,12 +47,26 @@ namespace Gobby
 class Window : public Gtk::Window
 {
 public:
-	Window();
+	Window(const IconManager& icon_mgr, Config& config);
 	~Window();
 
+	/** Offers a pointer to the currently visible document. If the user
+	 * is not subscribed to a document, NULL is returned.
+	 */
+	Document* get_current_document();
+
+	/** Opens a document containing the content of a file mounted on the
+	 * local filesystem.
+	 */
+	void open_local_file(const Glib::ustring& file);
+
+	/** Saves an existing document to the given path.
+	 */
+	void save_local_file(Document& doc, const Glib::ustring& file);
 protected:
 	// Gtk::Window overrides
 	virtual void on_realize();
+	virtual bool on_delete_event(GdkEventAny* event);
 
 	// Start/End obby session
 	void obby_start();
@@ -56,6 +75,7 @@ protected:
 	// Header UI handler
 	void on_session_create();
 	void on_session_join();
+	void on_session_save();
 	void on_session_quit();
 
 	void on_document_create();
@@ -64,6 +84,9 @@ protected:
 	void on_document_save_as();
 	void on_document_close();
 
+	void on_edit_search();
+	void on_edit_search_replace();
+	void on_edit_goto_line();
 	void on_edit_preferences();
 
 	void on_user_set_password();
@@ -73,69 +96,63 @@ protected:
 	void on_view_language(const Glib::RefPtr<Gtk::SourceLanguage>& lang);
 
 	// Folder UI handler
-	void on_folder_document_close(Document& document);
+	void on_folder_document_add(DocWindow& window);
+	void on_folder_document_remove(DocWindow& window);
+	void on_folder_document_close_request(Document& document);
 	void on_folder_tab_switched(Document& document);
 
 	void on_about();
 	void on_quit();
 
-	void on_chat(const Glib::ustring& message);
-
-	// Drag and drop
-	virtual void on_drag_data_received(
-		const Glib::RefPtr<Gdk::DragContext>& context,
-		int x, int y, const Gtk::SelectionData& data,
-		guint info, guint time
-	);
-
 	// Obby signal handlers
 	void on_obby_close();
 
-	void on_obby_user_join(obby::user& user);
-	void on_obby_user_part(obby::user& user);
-	void on_obby_user_colour(obby::user& user);
+	void on_obby_user_join(const obby::user& user);
+	void on_obby_user_part(const obby::user& user);
+	void on_obby_user_colour(const obby::user& user);
 	void on_obby_user_colour_failed();
 	void on_obby_document_insert(obby::document_info& document);
 	void on_obby_document_remove(obby::document_info& document);
 
-	void on_obby_server_chat(const Glib::ustring& message);
-	void on_obby_chat(obby::user& user, const Glib::ustring& message);
-
 	// Helper functions
-	Document& get_current_document();
 	void apply_preferences();
-	void update_title_bar(const Document& doc);
-	void open_local_file(const Glib::ustring& file);
-	void save_local_file(Document& doc, const Glib::ustring& file);
+	void update_title_bar();
 	void close_document(DocWindow& doc);
 	void display_error(const Glib::ustring& message,
 	                   const Gtk::MessageType type = Gtk::MESSAGE_ERROR);
 
 	// Config
-	Config m_config;
+	Config& m_config;
 	Preferences m_preferences;
+	const IconManager& m_icon_mgr;
 
 	// Paths
-	Glib::ustring m_last_path;
-	Glib::ustring m_local_file_path;
+	std::string m_last_path;
+	std::string m_local_file_path;
+	std::string m_prev_session;
 
 	// GUI
 	Gtk::VBox m_mainbox;
-
 	Gtk::VPaned m_mainpaned;
-	Gtk::HPaned m_subpaned;
 
 	Gtk::Frame m_frame_chat;
-	Gtk::Frame m_frame_list;
 	Gtk::Frame m_frame_text;
 
-	Folder m_folder;
-	UserList m_userlist;
-	Chat m_chat;
-
 	Header m_header;
+	UserList m_userlist;
+	DocumentList m_documentlist;
+
+	FindDialog m_finddialog;
+	GotoDialog m_gotodialog;
+
+	Folder m_folder;
+	Chat m_chat;
 	StatusBar m_statusbar;
 
+	/** Drag+Drop handler
+	 */
+	std::auto_ptr<DragDrop> m_dnd;
+
 	// obby
 	std::auto_ptr<obby::local_buffer> m_buffer;
 #ifdef WITH_HOWL
