@@ -16,13 +16,8 @@
  * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#include <iostream>
 #include <obby/format_string.hpp>
 #include <obby/user_table.hpp>
-#include <obby/client_document.hpp>
-#include <obby/host_document.hpp>
-#include <obby/client_buffer.hpp>
-#include <obby/host_buffer.hpp>
 #include "common.hpp"
 #include "document.hpp"
 #include "folder.hpp"
@@ -30,10 +25,14 @@
 Gobby::Document::Document(obby::local_document_info& doc, const Folder& folder,
                           const Preferences& preferences)
  : Gtk::SourceView(),
-   m_doc(doc), m_folder(folder), m_subscribed(false),
+   m_doc(doc),
    m_preferences(preferences), m_editing(true),
-   m_btn_subscribe(_("Subscribe") ), m_title(doc.get_title() )
+   m_title(doc.get_title() )
 {
+	// Documents can only show content if the local user is subscribed
+	if(!doc.is_subscribed() )
+		throw std::logic_error("Gobby::Document::Document");
+
 	Glib::RefPtr<Gtk::SourceBuffer> buf = get_buffer();
 
 	// Prevent from GTK sourceview's undo 
@@ -47,22 +46,32 @@ Gobby::Document::Document(obby::local_document_info& doc, const Folder& folder,
 	// Set SourceLanguage by file extension
 	Glib::ustring mime_type =
 		folder.get_mime_map().get_mime_type_by_file(doc.get_title() );
+
 	if(!mime_type.empty() )
 	{
-		Glib::RefPtr<Gtk::SourceLanguagesManager> manager =
+		Glib::RefPtr<const Gtk::SourceLanguagesManager> manager =
 			folder.get_lang_manager();
 		Glib::RefPtr<Gtk::SourceLanguage> language = 
 			manager->get_language_from_mime_type(mime_type);
 
-		if(language)
-			buf->set_language(language);
+		set_language(language);
 	}
 
+	// Switch syntax highlighting on
+	buf->set_highlight(true);
+
 	// Insert user tags into the tag table
 	const obby::user_table& user_table = doc.get_buffer().get_user_table();
-	for(obby::user_table::user_iterator<obby::user::NONE> iter =
-		user_table.user_begin<obby::user::NONE>();
-	    iter != user_table.user_end<obby::user::NONE>();
+	for(obby::user_table::iterator iter =
+		user_table.begin(
+			obby::user::flags::NONE,
+			obby::user::flags::NONE
+		);
+	    iter !=
+		user_table.end(
+			obby::user::flags::NONE,
+			obby::user::flags::NONE
+		);
 	    ++ iter)
 	{
 		// Create new tag
@@ -71,9 +80,9 @@ Gobby::Document::Document(obby::local_document_info& doc, const Folder& folder,
 
 		// Build user color
 		Gdk::Color color;
-		color.set_red(iter->get_red() * 65535 / 255);
-		color.set_green(iter->get_green() * 65535 / 255);
-		color.set_blue(iter->get_blue() * 65535 / 255);
+		color.set_red(iter->get_colour().get_red() * 65535 / 255);
+		color.set_green(iter->get_colour().get_green() * 65535 / 255);
+		color.set_blue(iter->get_colour().get_blue() * 65535 / 255);
 
 		// Assign color to tag
 		tag->property_background_gdk() = color;
@@ -93,38 +102,74 @@ Gobby::Document::Document(obby::local_document_info& doc, const Folder& folder,
 	buf->signal_apply_tag().connect(
 		sigc::mem_fun(*this, &Document::on_apply_tag_after), true);
 
-	// Obby signal handlers
-	doc.subscribe_event().connect(
-		sigc::mem_fun(*this, &Document::on_obby_user_subscribe) );
-	doc.unsubscribe_event().connect(
-		sigc::mem_fun(*this, &Document::on_obby_user_unsubscribe) );
-
-#if 0
-	// Allow drag+drop of uri-lists and plaintext. uri-list is forwarded to
-	// the window while plaintext will be inserted into the document.
-	std::list<Gtk::TargetEntry> targets;
-	targets.push_back(Gtk::TargetEntry("text/uri-list") );
-	targets.push_back(Gtk::TargetEntry("text/plain") );
-	drag_dest_set(targets);
-#endif
-
-	// GUI signal handlers
-	m_btn_subscribe.signal_clicked().connect(
-		sigc::mem_fun(*this, &Document::on_gui_subscribe) );
+	// Install signal handlers
+	const obby::document& content = doc.get_content();
+	content.insert_event().before().connect(
+		sigc::mem_fun(*this, &Document::on_obby_insert_before) );
+	content.insert_event().after().connect(
+		sigc::mem_fun(*this, &Document::on_obby_insert_after) );
+	content.delete_event().before().connect(
+		sigc::mem_fun(*this, &Document::on_obby_delete_before) );
+	content.delete_event().after().connect(
+		sigc::mem_fun(*this, &Document::on_obby_delete_after) );
 
 	// Apply preferences
 	apply_preferences();
 
-	// Set introduction text
-	set_intro_text();
+	// Set initial text
+	m_editing = true;
+	buf->set_text(content.get_text() );
+
+	// Not modified when subscribed, if the text is empty
+	buf->set_modified(
+		content.get_line_count() != 1 ||
+		content.get_line(0).length() != 0
+	);
+
+	// Set initial authors
+	for(unsigned int i = 0; i < content.get_line_count(); ++ i)
+	{
+		// Get current line
+		const obby::line& line = content.get_line(i);
+		obby::line::author_iterator prev = line.author_begin();
+		obby::line::author_iterator cur = prev;
+
+		// Iterate through it's authors list
+		for(++ cur; prev != line.author_end(); ++ cur)
+		{
+			// Get current user
+			const obby::user* user = prev->author;
+
+			// user can be NULL (server insert event, but then
+			// we do not have to apply a tag or so).
+			if(user == NULL) { prev = cur; continue; }
+
+			// Get the range to highlight
+			obby::line::size_type endpos;
+			if(cur != line.author_end() )
+				endpos = cur->position;
+			else
+				endpos = line.length();
+
+			Gtk::TextBuffer::iterator begin =
+				buf->get_iter_at_line_index(i, prev->position);
+			Gtk::TextBuffer::iterator end =
+				buf->get_iter_at_line_index(i, endpos);
+
+			// Apply corresponding tag
+			buf->apply_tag_by_name(
+				"gobby_user_" + user->get_name(),
+				begin,
+				end
+			);
+
+			prev = cur;
+		}
+	}
 
 	m_editing = false;
 }
 
-Gobby::Document::~Document()
-{
-}
-
 const obby::local_document_info& Gobby::Document::get_document() const
 {
 	return m_doc;
@@ -170,49 +215,35 @@ void Gobby::Document::get_cursor_position(unsigned int& row,
 	col = iter.get_line_offset();
 
 	// Add tab characters to col
-	if(m_doc.get_document() && is_subscribed() )
-	{
-		const std::string& line = m_doc.get_document()->get_line(row);
-		unsigned int tabs = m_preferences.editor.tab_width;
+	const std::string& line = m_doc.get_content().get_line(row);
+	unsigned int tabs = m_preferences.editor.tab_width;
 
-		// col chars
-		std::string::size_type chars = col; col = 0;
-		for(std::string::size_type i = 0; i < chars; ++ i)
+	// col chars
+	std::string::size_type chars = col; col = 0;
+	for(std::string::size_type i = 0; i < chars; ++ i)
+	{
+		unsigned int width = 1;
+		if(line[i] == '\t')
 		{
-			unsigned int width = 1;
-			if(line[i] == '\t')
-			{
-				width = (tabs - col % tabs) % tabs;
-				if(width == 0) width = tabs;
-			}
-			col += width;
+			width = (tabs - col % tabs) % tabs;
+			if(width == 0) width = tabs;
 		}
+		col += width;
 	}
 }
 
-unsigned int Gobby::Document::get_unsynced_changes_count() const
+void Gobby::Document::set_selection(const Gtk::TextIter& begin,
+                                    const Gtk::TextIter& end)
 {
-	// Get document
-	obby::local_document* local_doc = m_doc.get_document();
-	// No document? Seems that we are not subscribed
-	if(!local_doc) return 0;
-	// Cast to client document
-	obby::client_document* client_doc = 
-		dynamic_cast<obby::client_document*>(local_doc);
-	// No client document? Host is always synced
-	if(!client_doc) return 0;
-	// Document returns amount otherwise
-	return client_doc->unsynced_count();
+        get_buffer()->select_range(begin, end);
+        scroll_to(get_buffer()->get_insert(), 0.1);
 }
 
-unsigned int Gobby::Document::get_revision() const
+Glib::ustring Gobby::Document::get_selection_text() const
 {
-	// Get document
-	obby::local_document* local_doc = m_doc.get_document();
-	// No document? Seems that we are not subscribed (-> no revision)
-	if(!local_doc) return 0;
-	// Get revision from obby document
-	return local_doc->get_revision();
+	Gtk::TextIter start, end;
+	get_buffer()->get_selection_bounds(start, end);
+	return start.get_slice(end);
 }
 
 const Glib::ustring& Gobby::Document::get_title() const
@@ -225,11 +256,6 @@ const Glib::ustring& Gobby::Document::get_path() const
 	return m_path;
 }
 
-bool Gobby::Document::is_subscribed() const
-{
-	return m_subscribed;
-}
-
 bool Gobby::Document::get_modified() const
 {
 	return get_buffer()->get_modified();
@@ -269,21 +295,23 @@ Glib::ustring Gobby::Document::get_content()
 	return get_buffer()->get_text();
 }
 
-void Gobby::Document::obby_user_join(obby::user& user)
+void Gobby::Document::obby_user_join(const obby::user& user)
 {
 	update_tag_colour(user);
 }
 
-void Gobby::Document::obby_user_part(obby::user& user)
+void Gobby::Document::obby_user_part(const obby::user& user)
 {
 }
 
-void Gobby::Document::obby_user_colour(obby::user& user)
+void Gobby::Document::obby_user_colour(const obby::user& user)
 {
 	update_tag_colour(user);
 }
 
-void Gobby::Document::on_obby_insert_before(const obby::insert_record& record)
+void Gobby::Document::on_obby_insert_before(obby::position pos,
+                                            const std::string& text,
+                                            const obby::user* author)
 {
 	if(m_editing) return;
 	m_editing = true;
@@ -293,41 +321,52 @@ void Gobby::Document::on_obby_insert_before(const obby::insert_record& record)
 
 	// Translate position to row/column
 	unsigned int row, col;
-	m_doc.get_document()->position_to_coord(
-		record.get_position(), row, col);
-
-	// Find obby::user that inserted the text
-	const obby::user* user = record.get_user();
+	m_doc.get_content().position_to_coord(pos, row, col);
 
 	// Insert text
 	Gtk::TextBuffer::iterator end = buffer->insert(
-		buffer->get_iter_at_line_index(row, col),
-		record.get_text()
+		buffer->get_iter_at_line_index(row, col), text
 	);
 
-	// Colourize new text with that user's color
+	// Get iterator to beginning of inserted text
 	Gtk::TextBuffer::iterator begin = end;
-	begin.backward_chars(record.get_text().length() );
-	update_user_colour(begin, end, user);
+	begin.backward_chars(Glib::ustring(text).length() );
+
+	// Left gravity cursor ona remote inserts
+	if(end == buffer->get_insert()->get_iter() )
+		buffer->move_mark(buffer->get_insert(), begin);
+
+	if(end == buffer->get_selection_bound()->get_iter() )
+		buffer->move_mark(buffer->get_selection_bound(), begin);
+
+	// Colourize new text with that user's color
+	update_user_colour(begin, end, author);
 
 	m_editing = false;
 }
 
-void Gobby::Document::on_obby_insert_after(const obby::insert_record& record)
+void Gobby::Document::on_obby_insert_after(obby::position pos,
+                                           const std::string& text,
+                                           const obby::user* author)
 {
+	if(m_editing) return;
+
+	m_signal_cursor_moved.emit();
+	m_signal_content_changed.emit();
 }
 
-void Gobby::Document::on_obby_delete_before(const obby::delete_record& record)
+void Gobby::Document::on_obby_delete_before(obby::position pos,
+                                            obby::position len,
+                                            const obby::user* author)
 {
 	if(m_editing) return;
 	m_editing = true;
 
 	Glib::RefPtr<Gtk::TextBuffer> buffer = get_buffer();
 	unsigned int brow, bcol, erow, ecol;
-	obby::local_document* doc = m_doc.get_document();
 
-	doc->position_to_coord(record.get_begin(), brow, bcol);
-	doc->position_to_coord(record.get_end(), erow, ecol);
+	m_doc.get_content().position_to_coord(pos, brow, bcol);
+	m_doc.get_content().position_to_coord(pos + len, erow, ecol);
 
 	buffer->erase(
 		buffer->get_iter_at_line_index(brow, bcol),
@@ -337,170 +376,16 @@ void Gobby::Document::on_obby_delete_before(const obby::delete_record& record)
 	m_editing = false;
 }
 
-void Gobby::Document::on_obby_delete_after(const obby::delete_record& record)
-{
-}
-
-void Gobby::Document::on_obby_change_before()
-{
-}
-
-void Gobby::Document::on_obby_change_after()
+void Gobby::Document::on_obby_delete_after(obby::position pos,
+                                           obby::position len,
+                                           const obby::user* author)
 {
 	if(m_editing) return;
 
-	// Content may have changed / cursor have been moved
 	m_signal_cursor_moved.emit();
 	m_signal_content_changed.emit();
 }
 
-void Gobby::Document::on_obby_user_subscribe(const obby::user& user)
-{
-	// Call self function if the local user subscribed
-	if(&user == &m_doc.get_buffer().get_self() )
-		on_obby_self_subscribe();
-}
-
-void Gobby::Document::on_obby_user_unsubscribe(const obby::user& user)
-{
-	// Call self function if the local user unsubscribed
-	if(&user == &m_doc.get_buffer().get_self() )
-		on_obby_self_unsubscribe();
-}
-
-void Gobby::Document::on_obby_self_subscribe()
-{
-	// We are subscribed
-	m_subscribed = true;
-
-	// Get document we subscribed to
-	obby::local_document& doc = *m_doc.get_document();
-	Glib::RefPtr<Gtk::SourceBuffer> buf = get_buffer();
-
-	// Install signal handlers
-	doc.insert_event().before().connect(
-		sigc::mem_fun(*this, &Document::on_obby_insert_before) );
-	doc.insert_event().after().connect(
-		sigc::mem_fun(*this, &Document::on_obby_insert_after) );
-	doc.delete_event().before().connect(
-		sigc::mem_fun(*this, &Document::on_obby_delete_before) );
-	doc.delete_event().after().connect(
-		sigc::mem_fun(*this, &Document::on_obby_delete_after) );
-	doc.change_event().before().connect(
-		sigc::mem_fun(*this, &Document::on_obby_change_before) );
-	doc.change_event().after().connect(
-		sigc::mem_fun(*this, &Document::on_obby_change_after) );
-
-	// Set initial text
-	m_editing = true;
-	buf->set_text(doc.get_text() );
-
-	// Make the document editable
-	set_editable(true);
-
-	// Apply preferences
-	apply_preferences();
-
-	// Enable highlighting
-	buf->set_highlight(true);
-
-	// Not modified when subscribed
-	buf->set_modified(false);
-
-	// Set initial authors
-	for(unsigned int i = 0; i < doc.get_line_count(); ++ i)
-	{
-		// Get current line
-		const obby::line& line = doc.get_line(i);
-		obby::line::author_iterator prev = line.author_begin();
-		obby::line::author_iterator cur = prev;
-
-		// Iterate through it's authors list
-		for(++ cur; prev != line.author_end(); ++ cur)
-		{
-			// Get current user
-			const obby::user* user = prev->author;
-
-			// user can be NULL (server insert event, but then
-			// we do not have to apply a tag or so).
-			if(user == NULL) { prev = cur; continue; }
-
-			// Get the range to highlight
-			obby::line::size_type endpos;
-			if(cur != line.author_end() )
-				endpos = cur->position;
-			else
-				endpos = line.length();
-
-			Gtk::TextBuffer::iterator begin =
-				buf->get_iter_at_line_index(i, prev->position);
-			Gtk::TextBuffer::iterator end =
-				buf->get_iter_at_line_index(i, endpos);
-
-			// Apply corresponding tag
-			buf->apply_tag_by_name(
-				"gobby_user_" + user->get_name(),
-				begin,
-				end
-			);
-
-			prev = cur;
-		}
-	}
-
-	// Cursor moved because the introduction text has been deleted
-	m_signal_cursor_moved.emit();
-	// Content changed because the introduction text has been deleted
-	m_signal_content_changed.emit();
-
-	m_editing = false;
-}
-
-void Gobby::Document::on_obby_self_unsubscribe()
-{
-	// We are not subscribed anymore
-	m_subscribed = false;
-	// Prevet from execution of signal handlers
-	m_editing = true;
-	// Apply preferences (which may change for non-subscribed documents)
-	apply_preferences();
-	// Set introduction text
-	set_intro_text();
-	// Re-enable signal handlers
-	m_editing = false;
-}
-
-void Gobby::Document::on_gui_subscribe()
-{
-	// Send subscribe request
-	m_doc.subscribe();
-	// Deactivate the subscribe button since the request has been sent
-	m_btn_subscribe.set_sensitive(false);
-}
-
-#if 0
-// Hack to allow to drop files on a document. They will be opened as new
-// documents if the contained data is an uri list, inserted into this document
-// if its text.
-bool Gobby::Document::on_drag_motion(
-	const Glib::RefPtr<Gdk::DragContext>& context,
-	int x, int y, guint32 time
-)
-{
-	// Check available targets
-	std::vector<std::string> targets = context->get_targets();
-	for(unsigned int i = 0; i < targets.size(); ++ i)
-		// Is one of them uri-lists?
-		if(targets[i] == "text/uri-list")
-			// Yes so stop here to not show the insertion marker
-			// The event will be delayed to
-			return false;
-
-	// Call base function otherwise
-	return Gtk::SourceView::on_drag_motion(context, x, y, time);
-}
-#endif
-
 void Gobby::Document::on_insert_before(const Gtk::TextBuffer::iterator& begin,
                                        const Glib::ustring& text,
                                        int bytes)
@@ -508,9 +393,8 @@ void Gobby::Document::on_insert_before(const Gtk::TextBuffer::iterator& begin,
 	if(m_editing) return;
 	m_editing = true;
 
-	obby::local_document* doc = m_doc.get_document();
-	doc->insert(
-		doc->coord_to_position(
+	m_doc.insert(
+		m_doc.get_content().coord_to_position(
 			begin.get_line(),
 			begin.get_line_index()
 		),
@@ -526,15 +410,17 @@ void Gobby::Document::on_erase_before(const Gtk::TextBuffer::iterator& begin,
 	if(m_editing) return;
 	m_editing = true;
 
-	obby::local_document* doc = m_doc.get_document();
-	doc->erase(
-		doc->coord_to_position(
+	m_doc.erase(
+		m_doc.get_content().coord_to_position(
 			begin.get_line(),
 			begin.get_line_index()
 		),
-		doc->coord_to_position(
+		m_doc.get_content().coord_to_position(
 			end.get_line(),
 			end.get_line_index()
+		) - m_doc.get_content().coord_to_position(
+			begin.get_line(),
+			begin.get_line_index()
 		)
 	);
 
@@ -596,8 +482,7 @@ void Gobby::Document::on_apply_tag_after(const Glib::RefPtr<Gtk::TextTag>& tag,
 		unsigned int num_col = begin.get_line_index();
 
 		// Find author of the text
-		obby::local_document* doc = m_doc.get_document();
-		const obby::line& line = doc->get_line(num_line);
+		const obby::line& line = m_doc.get_content().get_line(num_line);
 
 		obby::line::author_iterator iter = line.author_begin();
 		for(iter; iter != line.author_end(); ++ iter)
@@ -653,49 +538,11 @@ void Gobby::Document::update_user_colour(const Gtk::TextBuffer::iterator& begin,
 	Glib::RefPtr<Gtk::TextTag> tag =
 		tag_table->lookup("gobby_user_" + user->get_name() );
 
-	// Apply tag twice to show it up immediately. I do not know why this
-	// is necessary but if we do only apply it once, we have to wait for
-	// the next event that refreshes the current line. I tried appending
-	// queue_draw() after the apply_tag call, but it did not help
-	// - Armin
-	for(int i = 0; i < 2; ++ i)
-		buffer->apply_tag(tag, begin, end);
-}
+	// Make sure the tag exists
+	if(!tag)
+		throw std::logic_error("Gobby::Document::update_user_colour");
 
-void Gobby::Document::set_intro_text()
-{
-	Glib::RefPtr<Gtk::SourceBuffer> buf = get_buffer();
-
-	// Build text
-	obby::format_string str(_(
-		"You are not subscribed to the document \"%0%\".\n\n"
-		"To view changes that others make or to edit the document "
-		"yourself, you have to subscribe to this document. Use the "
-		"following button to perform this.\n\n"
-	) );
-	str << m_doc.get_title();
-
-	// Set it
-	buf->set_text(str.str() );
-
-	// Add child anchor for the button
-	Glib::RefPtr<Gtk::TextChildAnchor> anchor =
-		buf->create_child_anchor(buf->end() );
-
-	// Activate the subscribe button, if it isn't
-	m_btn_subscribe.set_sensitive(true);
-
-	// Add the button to the anchor
-	add_child_at_anchor(m_btn_subscribe, anchor);
-
-	// TODO: Add people that are currently subscribed
-	set_editable(false);
-
-	// Intro text is not modified
-	get_buffer()->set_modified(false);
-
-	// Do not highlight anything until the user subscribed
-	buf->set_highlight(false);
+	buffer->apply_tag(tag, begin, end);
 }
 
 void Gobby::Document::apply_preferences()
@@ -707,25 +554,17 @@ void Gobby::Document::apply_preferences()
 	set_smart_home_end(m_preferences.editor.homeend_smart);
 
 	// View
-	if(m_subscribed)
+	// Check preference for wrapped text
+	if(m_preferences.view.wrap_text)
 	{
-		// Check preference for wrapped text
-		if(m_preferences.view.wrap_text)
-		{
-			if(m_preferences.view.wrap_words)
-				set_wrap_mode(Gtk::WRAP_CHAR);
-			else
-				set_wrap_mode(Gtk::WRAP_WORD);
-		}
+		if(m_preferences.view.wrap_words)
+			set_wrap_mode(Gtk::WRAP_CHAR);
 		else
-		{
-			set_wrap_mode(Gtk::WRAP_NONE);
-		}
+			set_wrap_mode(Gtk::WRAP_WORD);
 	}
 	else
 	{
-		// Wrap when not subscribed (intro text)
-		set_wrap_mode(Gtk::WRAP_WORD_CHAR);
+		set_wrap_mode(Gtk::WRAP_NONE);
 	}
 
 	set_show_line_numbers(m_preferences.view.linenum_display);
@@ -749,7 +588,7 @@ Gobby::Document::on_remove_user_colour(Glib::RefPtr<Gtk::TextBuffer::Tag> tag,
 		get_buffer()->remove_tag(tag, begin, end);
 }
 
-void Gobby::Document::update_tag_colour(obby::user& user)
+void Gobby::Document::update_tag_colour(const obby::user& user)
 {
 	// Build tag name for this user
 	Glib::ustring tag_name = "gobby_user_" + user.get_name();
@@ -766,11 +605,10 @@ void Gobby::Document::update_tag_colour(obby::user& user)
 
 	// Build color
 	Gdk::Color color;
-	color.set_red(user.get_red() * 65535 / 255);
-	color.set_green(user.get_green() * 65535 / 255);
-	color.set_blue(user.get_blue() * 65535 / 255);
+	color.set_red(user.get_colour().get_red() * 65535 / 255);
+	color.set_green(user.get_colour().get_green() * 65535 / 255);
+	color.set_blue(user.get_colour().get_blue() * 65535 / 255);
 
 	// Set/Update color
 	tag->property_background_gdk() = color;
 }
-
