@@ -20,6 +20,8 @@
 #include <gtkmm/main.h>
 #include <gtkmm/messagedialog.h>
 #include "common.hpp"
+#include "icon.hpp"
+#include "config.hpp"
 #include "window.hpp"
 #include "features.hpp"
 
@@ -34,14 +36,23 @@ void handle_exception(const Glib::ustring& message)
 int main(int argc, char* argv[]) try
 {
 	setlocale(LC_ALL, "");
-	bindtextdomain(GETTEXT_PACKAGE, LOCALE_DIR);
-	bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
-	textdomain(GETTEXT_PACKAGE);
+	net6::gettext_package gobby_package(GETTEXT_PACKAGE, LOCALE_DIR);
+	Gobby::init_gettext(gobby_package);
 
 	Gtk::Main kit(argc, argv);
 	Glib::thread_init();
 
-	Gobby::Window wnd;
+	Gobby::IconManager icon_mgr;
+
+	// Set default icon
+	std::list<Glib::RefPtr<Gdk::Pixbuf> > icon_list;
+	icon_list.push_back(icon_mgr.gobby);
+	Gtk::Window::set_default_icon_list(icon_list);
+
+	// Read the configuration
+	Gobby::Config config(Glib::get_home_dir() + "/.gobby/config.xml");
+
+	Gobby::Window wnd(icon_mgr, config);
 	wnd.show_all();
 	kit.run(wnd);
 	return 0;
