@@ -16,10 +16,12 @@
  * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+#include <cerrno>
 #include <stdexcept>
 #include <fstream>
 #include <ostream>
 
+#include <glibmm/miscutils.h>
 #include <gtkmm/main.h>
 #include <gtkmm/aboutdialog.h>
 #include <gtkmm/messagedialog.h>
@@ -45,78 +47,92 @@
 #include "icon.hpp"
 #include "colorsel.hpp"
 
-Gobby::Window::Window()
- : Gtk::Window(Gtk::WINDOW_TOPLEVEL), 
-   m_config(Glib::get_home_dir() + "/.gobby/config.xml"),
-   m_preferences(m_config), m_buffer(NULL),
+Gobby::Window::Window(const IconManager& icon_mgr, Config& config)
+ : Gtk::Window(Gtk::WINDOW_TOPLEVEL),
+   m_config(config),
+   m_preferences(m_config), m_icon_mgr(icon_mgr),
+   m_buffer(NULL),
 #ifdef WITH_HOWL
    m_zeroconf(NULL),
 #endif
-   m_folder(m_preferences), m_userlist(m_folder),
-   m_header(m_folder), m_statusbar(m_folder)
+   m_header(),
+   m_userlist(*this, m_header, m_preferences, config["windows"]),
+   m_documentlist(*this, m_header, m_preferences, config["windows"]),
+   m_finddialog(*this), m_gotodialog(*this),
+   m_folder(m_header, m_preferences),
+   m_statusbar(m_header, m_folder)
 {
 	// Header
-	m_header.session_create_event().connect(
+	m_header.action_app_session_create->signal_activate().connect(
 		sigc::mem_fun(*this, &Window::on_session_create) );
-	m_header.session_join_event().connect(
+	m_header.action_app_session_join->signal_activate().connect(
 		sigc::mem_fun(*this, &Window::on_session_join) );
-	m_header.session_quit_event().connect(
+	m_header.action_app_session_save->signal_activate().connect(
+		sigc::mem_fun(*this, &Window::on_session_save) );
+	m_header.action_app_session_quit->signal_activate().connect(
 		sigc::mem_fun(*this, &Window::on_session_quit) );
-
-	m_header.document_create_event().connect(
-		sigc::mem_fun(*this, &Window::on_document_create) );
-	m_header.document_open_event().connect(
-		sigc::mem_fun(*this, &Window::on_document_open) );
-	m_header.document_save_event().connect(
-		sigc::mem_fun(*this, &Window::on_document_save) );
-	m_header.document_save_as_event().connect(
-		sigc::mem_fun(*this, &Window::on_document_save_as) );
-	m_header.document_close_event().connect(
-		sigc::mem_fun(*this, &Window::on_document_close) );
-
-	m_header.edit_preferences_event().connect(
-		sigc::mem_fun(*this, &Window::on_edit_preferences) );
-
-	m_header.user_set_password_event().connect(
-		sigc::mem_fun(*this, &Window::on_user_set_password) );
-	m_header.user_set_colour_event().connect(
-		sigc::mem_fun(*this, &Window::on_user_set_colour) );
-	
-	m_header.view_preferences_event().connect(
-		sigc::mem_fun(*this, &Window::on_view_preferences) );
-	m_header.view_language_event().connect(
-		sigc::mem_fun(*this, &Window::on_view_language) );
-
-	m_header.about_event().connect(
-		sigc::mem_fun(*this, &Window::on_about) );
-	m_header.quit_event().connect(
+	m_header.action_app_quit->signal_activate().connect(
 		sigc::mem_fun(*this, &Window::on_quit) );
 
+	m_header.action_session_document_create->signal_activate().connect(
+		sigc::mem_fun(*this, &Window::on_document_create) );
+	m_header.action_session_document_open->signal_activate().connect(
+		sigc::mem_fun(*this, &Window::on_document_open) );
+	m_header.action_session_document_save->signal_activate().connect(
+		sigc::mem_fun(*this, &Window::on_document_save) );
+	m_header.action_session_document_save_as->signal_activate().connect(
+		sigc::mem_fun(*this, &Window::on_document_save_as) );
+	m_header.action_session_document_close->signal_activate().connect(
+		sigc::mem_fun(*this, &Window::on_document_close) );
+
+	m_header.action_edit_search->signal_activate().connect(
+		sigc::mem_fun(*this, &Window::on_edit_search) );
+	m_header.action_edit_search_replace->signal_activate().connect(
+		sigc::mem_fun(*this, &Window::on_edit_search_replace) );
+	m_header.action_edit_goto_line->signal_activate().connect(
+		sigc::mem_fun(*this, &Window::on_edit_goto_line) );
+	m_header.action_edit_preferences->signal_activate().connect(
+		sigc::mem_fun(*this, &Window::on_edit_preferences) );
+
+	m_header.action_user_set_password->signal_activate().connect(
+		sigc::mem_fun(*this, &Window::on_user_set_password) );
+	m_header.action_user_set_colour->signal_activate().connect(
+		sigc::mem_fun(*this, &Window::on_user_set_colour) );
+	
+	m_header.action_view_preferences->signal_activate().connect(
+		sigc::mem_fun(*this, &Window::on_view_preferences) );
+
+	m_header.action_help_about->signal_activate().connect(
+		sigc::mem_fun(*this, &Window::on_about) );
+
 	// Folder
-	m_folder.document_close_event().connect(
-		sigc::mem_fun(*this, &Window::on_folder_document_close) );
+	m_folder.document_add_event().connect(
+		sigc::mem_fun(*this, &Window::on_folder_document_add) );
+	m_folder.document_remove_event().connect(
+		sigc::mem_fun(*this, &Window::on_folder_document_remove) );
+	m_folder.document_close_request_event().connect(
+		sigc::mem_fun(*this, &Window::on_folder_document_close_request) );
 	m_folder.tab_switched_event().connect(
 		sigc::mem_fun(*this, &Window::on_folder_tab_switched) );
 
 	// Build UI
 	add_accel_group(m_header.get_accel_group() );
 
-	m_chat.chat_event().connect(
-		sigc::mem_fun(*this, &Window::on_chat) );
+	// Initialise header sensitivity
+	m_header.group_session->set_sensitive(false);
+	m_header.group_user->set_sensitive(false);
+	m_header.group_view->set_sensitive(false);
+	m_header.action_app_session_save->set_sensitive(false);
+	m_header.action_app_session_quit->set_sensitive(false);
 
 	m_frame_chat.set_shadow_type(Gtk::SHADOW_IN);
-	m_frame_list.set_shadow_type(Gtk::SHADOW_IN);
 	m_frame_text.set_shadow_type(Gtk::SHADOW_IN);
 
 	m_frame_chat.add(m_chat);
-	m_frame_list.add(m_userlist);
 	m_frame_text.add(m_folder);
 
-	m_subpaned.pack1(m_frame_text, true, false);
-	m_subpaned.pack2(m_frame_list, true, false);
-
 	m_mainpaned.set_border_width(10);
-	m_mainpaned.pack1(m_subpaned, true, false);
+	m_mainpaned.pack1(m_frame_text, true, false);
 	m_mainpaned.pack2(m_frame_chat, true, false);
 
 	m_mainbox.pack_start(m_header, Gtk::PACK_SHRINK);
@@ -146,6 +162,26 @@ Gobby::Window::Window()
 		m_zeroconf.reset();
 	}
 #endif
+
+	if(m_preferences.appearance.remember)
+	{
+		// Restore the window's position from the configuration
+        	const int x = config["windows"]["main"]["x"].get<int>(0);
+	        const int y = config["windows"]["main"]["y"].get<int>(0);
+		const int w = config["windows"]["main"]["width"].get<int>(0);
+		const int h = config["windows"]["main"]["height"].get<int>(0);
+		const int s_w = config["screen"]["width"].get<int>(0);
+		const int s_h = config["screen"]["height"].get<int>(0);
+		bool first_run = (x == 0 && y == 0 && w == 0 && h == 0);
+
+		Glib::RefPtr<Gdk::Screen> screen(get_screen() );
+		if((screen->get_width() >= s_w && screen->get_height() >= s_h)
+			&& !first_run)
+		{
+			move(x, y);
+			resize(w, h);
+		}
+	}
 }
 
 Gobby::Window::~Window()
@@ -154,6 +190,21 @@ Gobby::Window::~Window()
 
 	// Serialise preferences into config
 	m_preferences.serialise(m_config);
+
+	// Save the window's current position
+	if(m_preferences.appearance.remember)
+	{
+		int x, y, w, h;
+		get_position(x, y); get_size(w, h);
+		Glib::RefPtr<Gdk::Screen> screen(get_screen() );
+
+		m_config["windows"]["main"]["x"].set(x);
+		m_config["windows"]["main"]["y"].set(y);
+		m_config["windows"]["main"]["width"].set(w);
+		m_config["windows"]["main"]["height"].set(h);
+		m_config["screen"]["width"].set(screen->get_width() );
+		m_config["screen"]["height"].set(screen->get_height() );
+	}
 }
 
 void Gobby::Window::on_realize()
@@ -162,15 +213,35 @@ void Gobby::Window::on_realize()
 
 	// Initialize paned sizes. This cannot be done in the constructor
 	// because the widget's sizes are not known.
-	int submin = m_subpaned.property_min_position();
-	int submax = m_subpaned.property_max_position();
 	int mainmin = m_mainpaned.property_min_position();
 	int mainmax = m_mainpaned.property_max_position();
 
-	m_subpaned.set_position(submin + (submax - submin) * 4 / 5);
 	m_mainpaned.set_position(mainmin + (mainmax - mainmin) * 3 / 4);
 }
 
+bool Gobby::Window::on_delete_event(GdkEventAny* event)
+{
+	if(m_buffer.get() == NULL) return false;
+
+	Gtk::MessageDialog dlg(
+		*this,
+		_("You are still connected to a session"),
+		false,
+		Gtk::MESSAGE_WARNING,
+		Gtk::BUTTONS_NONE,
+		true
+	);
+
+	dlg.set_secondary_text(
+		_("Do you want to close Gobby nevertheless?")
+	);
+
+	dlg.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
+	dlg.add_button(Gtk::Stock::CLOSE, Gtk::RESPONSE_YES)->grab_focus();
+
+	return dlg.run() != Gtk::RESPONSE_YES;
+}
+
 void Gobby::Window::obby_start()
 {
 	// Connect to obby events
@@ -188,28 +259,36 @@ void Gobby::Window::obby_start()
 	m_buffer->document_remove_event().connect(
 		sigc::mem_fun(*this, &Window::on_obby_document_remove) );
 
-	m_buffer->message_event().connect(
-		sigc::mem_fun(*this, &Window::on_obby_chat) );
-	m_buffer->server_message_event().connect(
-		sigc::mem_fun(*this, &Window::on_obby_server_chat) );
-
 	// Accept drag and drop of files into the gobby window
-	std::list<Gtk::TargetEntry> targets;
-	targets.push_back(Gtk::TargetEntry("text/uri-list") );
-	drag_dest_set(targets);
+	m_dnd.reset(new DragDrop(*this) );
+
+	m_header.group_session->set_sensitive(true);
+	m_header.action_session_document_create->set_sensitive(true);
+	m_header.action_session_document_open->set_sensitive(true);
+	m_header.group_user->set_sensitive(true);
+	m_header.group_view->set_sensitive(true);
+	m_header.action_app_session_save->set_sensitive(true);
+	m_header.action_app_session_quit->set_sensitive(true);
+
+	m_header.action_app_session_create->set_sensitive(false);
+	m_header.action_app_session_join->set_sensitive(false);
+
+	m_header.action_user_set_password->set_sensitive(
+		dynamic_cast<obby::client_buffer*>(m_buffer.get()) != NULL
+	);
 
 	// Delegate start of obby session
-	m_header.obby_start(*m_buffer);
 	m_folder.obby_start(*m_buffer);
 	m_userlist.obby_start(*m_buffer);
+	m_documentlist.obby_start(*m_buffer);
 	m_chat.obby_start(*m_buffer);
 	m_statusbar.obby_start(*m_buffer);
 
-	// Forward user joins for users that are connected 
-	const obby::user_table& user_table = m_buffer->get_user_table();
-	for(obby::user_table::user_iterator<obby::user::CONNECTED> iter =
-		user_table.user_begin<obby::user::CONNECTED>();
-	    iter != user_table.user_end<obby::user::CONNECTED>();
+	// Forward user joins
+	const obby::user_table& table = m_buffer->get_user_table();
+	for(obby::user_table::iterator iter =
+		table.begin(obby::user::flags::NONE, obby::user::flags::NONE);
+	    iter != table.end(obby::user::flags::NONE, obby::user::flags::NONE);
 	    ++ iter)
 	{
 		on_obby_user_join(*iter);
@@ -223,28 +302,51 @@ void Gobby::Window::obby_start()
 	// Set last page as active one because it is currently shown anyway.
 	if(m_buffer->document_count() > 0)
 		m_folder.set_current_page(m_buffer->document_count() - 1);
+
+	// Clear location of previous session file, this is a new session
+	m_prev_session = "";
+
+	// Current document has changed, update titlebar
+	update_title_bar();
+
+	// Show up document list if obby buffer contains documents
+	if(m_buffer->document_count() > 0)
+	{
+		m_documentlist.show();
+		m_documentlist.grab_focus();
+	}
 }
 
 void Gobby::Window::obby_end()
 {
-	// No drag and drop anymore
-	drag_dest_unset();
-
 	// Nothing to do if no buffer is open
 	if(!m_buffer.get() ) return;
 
+	// Remove DND handler
+	m_dnd.reset(NULL);
+
 	// Tell GUI components that the session ended
-	m_header.obby_end();
 	m_folder.obby_end();
 	m_userlist.obby_end();
+	m_documentlist.obby_end();
 	m_chat.obby_end();
 	m_statusbar.obby_end();
 
+	m_header.action_session_document_create->set_sensitive(false);
+	m_header.action_session_document_open->set_sensitive(false);
+	m_header.group_user->set_sensitive(false);
+	m_header.action_app_session_save->set_sensitive(false);
+	m_header.action_app_session_quit->set_sensitive(false);
+
+	m_header.action_app_session_create->set_sensitive(true);
+	m_header.action_app_session_join->set_sensitive(true);
+
 	// Delete buffer
 	m_buffer.reset();
 
 #ifdef WITH_HOWL
-	m_zeroconf->unpublish_all();
+	if(m_zeroconf.get() )
+		m_zeroconf->unpublish_all();
 #endif
 }
 
@@ -261,9 +363,11 @@ void Gobby::Window::on_session_create()
 		Glib::ustring name = dlg.get_name();
 		Gdk::Color color = dlg.get_color();	
 		Glib::ustring password = dlg.get_password();
+		Glib::ustring session = dlg.get_session();
 
 		// Set up host with hostprogressdialog
-		HostProgressDialog prgdlg(*this, m_config, port, name, color);
+		HostProgressDialog prgdlg(
+			*this, m_config, port, name, color, session);
 		if(prgdlg.run() == Gtk::RESPONSE_OK)
 		{
 			prgdlg.hide();
@@ -281,9 +385,16 @@ void Gobby::Window::on_session_create()
 				m_zeroconf->publish(name, port);
 #endif
 
-			// Start session
+			obby::format_string str(_("Serving on port %0%") );
+			str << port;
+			m_statusbar.update_connection(str.str() );
+
 			m_buffer = buffer;
+
+			// Start session
 			obby_start();
+			// Remember session file
+			m_prev_session = Glib::filename_from_utf8(session);
 		}
 	}
 }
@@ -319,6 +430,10 @@ void Gobby::Window::on_session_join()
 			buffer->close_event().connect(
 				sigc::mem_fun(*this, &Window::on_obby_close) );
 
+			obby::format_string str(_("Connected to %0%:%1%") );
+			str << host << port;
+			m_statusbar.update_connection(str.str() );
+
 			// Start session
 			m_buffer = buffer;
 			obby_start();
@@ -326,6 +441,67 @@ void Gobby::Window::on_session_join()
 	}
 }
 
+void Gobby::Window::on_session_save()
+{
+	Gtk::CheckButton m_chk_default_ext(
+		_("Use default .obby extension if none is given")
+	);
+
+	Gtk::FileChooserDialog dlg(
+		*this,
+		_("Save obby session"),
+		Gtk::FILE_CHOOSER_ACTION_SAVE
+	);
+
+	m_chk_default_ext.set_active(true);
+	dlg.get_vbox()->pack_start(m_chk_default_ext, Gtk::PACK_SHRINK);
+	// This option confuses the overwrite confirmation :/
+//	m_chk_default_ext.show();
+
+#ifdef GTKMM_GEQ_28
+	dlg.set_do_overwrite_confirmation(true);
+#endif
+
+	// Use the location of a previously saved session, if any
+	if(!m_prev_session.empty() )
+	{
+		dlg.set_filename(m_prev_session);
+	}
+	else
+	{
+		// Use the last used path for this dialog, if we have any
+		if(!m_last_path.empty() )
+			dlg.set_current_folder(m_last_path);
+	}
+
+	dlg.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
+	dlg.add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);
+
+	if(dlg.run() == Gtk::RESPONSE_OK)
+	{
+		// Use current folder as standard folder for other dialogs
+		m_last_path = dlg.get_current_folder();
+		// Get selected filename
+		std::string filename = dlg.get_filename();
+
+		// Append .obby extension if none is given
+		/*if(m_chk_default_ext.get_active() )
+			if(filename.find('.') == std::string::npos)
+				filename += ".obby";*/
+
+		// Save document
+		try
+		{
+			m_buffer->serialise(filename);
+			m_prev_session = filename;
+		}
+		catch(std::exception& e)
+		{
+			display_error(e.what() );
+		}
+	}
+}
+
 void Gobby::Window::on_session_quit()
 {
 	obby_end();
@@ -338,12 +514,7 @@ void Gobby::Window::on_about()
 	dlg.set_version(PACKAGE_VERSION);
 	dlg.set_comments(_("A collaborative text editor"));
 	dlg.set_copyright("Copyright (C) 2005 0x539 dev group <crew@0x539.de>");
-#ifndef WITH_GNOME
-	dlg.set_logo(Gdk::Pixbuf::create_from_inline(512 * 128, Icon::gobby) );
-#else
-	dlg.set_logo(Gdk::Pixbuf::create_from_file(
-		PIXMAPS_DIR"/gobby.png") );
-#endif
+	dlg.set_logo(m_icon_mgr.gobby);
 
 	std::deque<Glib::ustring> authors;
 	authors.push_back("Developers:");
@@ -359,6 +530,30 @@ void Gobby::Window::on_about()
 	dlg.set_authors(authors);
 	dlg.set_artists(artists);
 
+	dlg.set_translator_credits(
+		"French translation:\n"
+		"  Peer Janssen <peer@baden-online.de>\n"
+		"  Mohammed Adnene Trojette <adn@diwi.org>\n"
+		"\n"
+		"Spanish translation:\n"
+		"  Mario Palomo Torrero <mpalomo@ihman.com>\n"
+		"\n"
+		"Swedish translation:\n"
+		"  Daniel Nylander <po@danielnylander.se>\n"
+		"\n"
+		"German translation:\n"
+		"  Philipp Kern <phil@0x539.de>\n"
+		"  Armin Burgmeier <armin@0x539.de>\n"
+		"\n"
+		"Please help in translating this release using Rosetta:\n"
+		"\n"
+		"After you create your account on https://launchpad.net\n"
+		"you could access the translation template on\n"
+		"https://launchpad.net/products/gobby/+series/0.3/+pots/gobby/"
+		"\nwhere you could translate it directly into your favourite\n"
+		"language. Your help is much appreciated!"
+		);
+
 	dlg.set_license(
 		"This program is free software; you can redistribute it\n"
 		"and/or modify it under the terms of the GNU General Public\n"
@@ -374,14 +569,56 @@ void Gobby::Window::on_about()
 	dlg.run();
 }
 
-void Gobby::Window::on_folder_document_close(Document& document)
+void Gobby::Window::on_folder_document_add(DocWindow& window)
 {
-	// TODO: Folder sollte eher Signale mit DocWindows emitten und so.
+	const obby::document_info& document =
+		window.get_document().get_document();
+
+	// Set the path from which this document was opened, if we opened that
+	// file.
+	if(document.get_owner() == &m_buffer->get_self() &&
+	   !m_local_file_path.empty() )
+	{
+		// " " is newly created, so we do not need a path
+		if(m_local_file_path != " ")
+			window.get_document().set_path(m_local_file_path);
+
+		// Crear local path
+		m_local_file_path.clear();
+	}
+
+	// Select newly created page
+	m_folder.set_current_page(m_folder.page_num(window) );
+	window.get_document().grab_focus();
+}
+
+void Gobby::Window::on_folder_document_remove(DocWindow& window)
+{
+	// Update title bar if there are no more documents left
+	// (folder_tab_switched is not emitted in this case)
+	if(m_folder.get_n_pages() == 0)
+	{
+		update_title_bar();
+
+		// TODO: This should be done automatically with flags for the
+		// menu items which specify when the entry is visible
+		if(m_buffer.get() == NULL)
+		{
+			m_header.group_session->set_sensitive(false);
+			m_header.group_view->set_sensitive(false);
+		}
+	}
+}
+
+void Gobby::Window::on_folder_document_close_request(Document& document)
+{
+	// TODO: Folder should emit signal with DocWindow
 	for(int i = 0; i < m_folder.get_n_pages(); ++ i)
 	{
 		Gtk::Widget* doc = m_folder.get_nth_page(i);
 		if(&static_cast<DocWindow*>(doc)->get_document() == &document)
 		{
+			// Close this document
 			close_document(*static_cast<DocWindow*>(doc));
 			break;
 		}
@@ -391,7 +628,7 @@ void Gobby::Window::on_folder_document_close(Document& document)
 void Gobby::Window::on_folder_tab_switched(Document& document)
 {
 	// Update title bar
-	update_title_bar(document);
+	update_title_bar();
 }
 
 void Gobby::Window::on_document_create()
@@ -404,7 +641,7 @@ void Gobby::Window::on_document_create()
 		// " " means a newly created file
 		m_local_file_path = " ";
 		// Create new document
-		m_buffer->create_document(dlg.get_text() );
+		m_buffer->document_create(dlg.get_text(), "");
 	}
 }
 
@@ -441,12 +678,14 @@ void Gobby::Window::on_document_open()
 void Gobby::Window::on_document_save()
 {
 	// Get page
-	Document& doc = get_current_document();
+	Document* doc = get_current_document();
+	if(doc == NULL)
+		throw std::logic_error("Gobby::Window::on_document_save");
 
 	// Is there already a path for this document?
-	if(!doc.get_path().empty() )
+	if(!doc->get_path().empty() )
 		// Yes, so save the document there
-		save_local_file(doc, doc.get_path() );
+		save_local_file(*doc, doc->get_path() );
 	else
 		// Open save as dialog otherwise
 		on_document_save_as();
@@ -455,17 +694,23 @@ void Gobby::Window::on_document_save()
 void Gobby::Window::on_document_save_as()
 {
 	// Get page
-	Document& doc = get_current_document();
+	Document* doc = get_current_document();
+	if(doc == NULL)
+		throw std::logic_error("Gobby::Window::on_document_save_as");
 
 	// Setup dialog
 	Gtk::FileChooserDialog dlg(*this, _("Save current document"),
 		Gtk::FILE_CHOOSER_ACTION_SAVE);
 
+#ifdef GTKMM_GEQ_28
+	dlg.set_do_overwrite_confirmation(true);
+#endif
+
 	// Does the document have already a path?
-	if(!doc.get_path().empty() )
+	if(!doc->get_path().empty() )
 	{
 		// Yes, so set it as filename
-		dlg.set_filename(doc.get_path() );
+		dlg.set_filename(doc->get_path() );
 	}
 	else
 	{
@@ -473,7 +718,7 @@ void Gobby::Window::on_document_save_as()
 		if(!m_last_path.empty() )
 			dlg.set_current_folder(m_last_path);
 		// Set current title as proposed file name
-		dlg.set_current_name(doc.get_title() );
+		dlg.set_current_name(doc->get_title() );
 	}
 
 	// Add buttons to close the dialog
@@ -485,7 +730,7 @@ void Gobby::Window::on_document_save_as()
 		// Use current folder as standard folder for other dialogs
 		m_last_path = dlg.get_current_folder();
 		// Save document
-		save_local_file(doc, dlg.get_filename() );
+		save_local_file(*doc, dlg.get_filename() );
 	}
 }
 
@@ -497,6 +742,26 @@ void Gobby::Window::on_document_close()
 	close_document(*static_cast<DocWindow*>(page) );
 }
 
+void Gobby::Window::on_edit_search()
+{
+	m_finddialog.set_search_only(true);
+	m_finddialog.show();
+	m_finddialog.grab_focus();
+}
+
+void Gobby::Window::on_edit_search_replace()
+{
+	m_finddialog.set_search_only(false);
+	m_finddialog.show();
+	m_finddialog.grab_focus();
+}
+
+void Gobby::Window::on_edit_goto_line()
+{
+	m_gotodialog.show();
+	m_gotodialog.grab_focus();
+}
+
 void Gobby::Window::on_edit_preferences()
 {
 	PreferencesDialog dlg(*this, m_preferences, false);
@@ -562,12 +827,12 @@ void Gobby::Window::on_user_set_colour()
 {
 	// Simple ColorSelectionDialog
 	ColorSelectionDialog dlg;
-	obby::user& user = m_buffer->get_self();
+	const obby::user& user = m_buffer->get_self();
 	Gdk::Color color;
 
-	color.set_red(user.get_red() * 65535 / 255);
-	color.set_green(user.get_green() * 65535 / 255);
-	color.set_blue(user.get_blue() * 65535 / 255);
+	color.set_red(user.get_colour().get_red() * 65535 / 255);
+	color.set_green(user.get_colour().get_green() * 65535 / 255);
+	color.set_blue(user.get_colour().get_blue() * 65535 / 255);
 	dlg.get_colorsel()->set_current_color(color);
 
 	// Run it
@@ -576,20 +841,24 @@ void Gobby::Window::on_user_set_colour()
 		// Convert GDK color to obby color, set new color
 		Gdk::Color color = dlg.get_colorsel()->get_current_color();
 		m_buffer->set_colour(
-			color.get_red() * 255 / 65535,
-			color.get_green() * 255 / 65535,
-			color.get_blue() * 255 / 65535
-			);
+			obby::colour(
+				color.get_red() * 255 / 65535,
+				color.get_green() * 255 / 65535,
+				color.get_blue() * 255 / 65535
+			)
+		);
 	}
 }
 
 void Gobby::Window::on_view_preferences()
 {
 	// Get current page
-	Document& doc = get_current_document();
+	Document* doc = get_current_document();
+	if(doc == NULL)
+		throw std::logic_error("Gobby::Window::on_view_preferences");
 
 	// Add preferences dialog
-	PreferencesDialog dlg(*this, doc.get_preferences(), true);
+	PreferencesDialog dlg(*this, doc->get_preferences(), true);
 
 	// Label text
 	obby::format_string str(_(
@@ -599,7 +868,7 @@ void Gobby::Window::on_view_preferences()
 	) );
 
 	// Get title
-	str << doc.get_title();
+	str << doc->get_title();
 
 	// Info label
 	Gtk::Label m_lbl_info(str.str() );
@@ -614,7 +883,7 @@ void Gobby::Window::on_view_preferences()
 	if(dlg.run() == Gtk::RESPONSE_OK)
 	{
 		// Apply new preferences to the document
-		doc.set_preferences(dlg.preferences() );
+		doc->set_preferences(dlg.preferences() );
 	}
 }
 
@@ -622,58 +891,22 @@ void
 Gobby::Window::on_view_language(const Glib::RefPtr<Gtk::SourceLanguage>& lang)
 {
 	// Set language of current document
-	get_current_document().set_language(lang);
+	Document* doc = get_current_document();
+	if(doc == NULL)
+		throw std::logic_error("Gobby::Window::on_view_language");
+
+	doc->set_language(lang);
 }
 
 void Gobby::Window::on_quit()
 {
-	// Quit session
-	obby_end();
-	// End program
-	Gtk::Main::quit();
-}
-
-void Gobby::Window::on_chat(const Glib::ustring& message)
-{
-	// Send chat message via obby
-	m_buffer->send_message(message);
-}
-
-/* Drag and Drop */
-void Gobby::Window::on_drag_data_received(
-	const Glib::RefPtr<Gdk::DragContext>& context,
-	int x, int y, const Gtk::SelectionData& data,
-	guint info, guint time
-)
-{
-	// We only accept uri-lists as new files to open
-	if(data.get_target() == "text/uri-list")
+	if(on_delete_event(NULL) == false)
 	{
-		// Get files by dragdata
-		std::vector<std::string> files = data.get_uris();
-		//std::unique(files.begin(), files.end() );
-
-		// Open all of them
-		for(std::vector<std::string>::iterator iter = files.begin();
-		    iter != files.end();
-		    ++ iter)
-		{
-			try
-			{
-				// Convert URI to filename
-				open_local_file(
-					Glib::filename_from_uri(*iter) );
-			}
-			catch(Glib::ConvertError& e)
-			{
-				// Show any errors while converting a file
-				display_error(e.what() );
-			}
-		}
+		// Quit session
+		obby_end();
+		// End program
+		Gtk::Main::quit();
 	}
-
-	// Call base function
-	Gtk::Window::on_drag_data_received(context, x, y, data, info, time);
 }
 
 void Gobby::Window::on_obby_close()
@@ -682,41 +915,30 @@ void Gobby::Window::on_obby_close()
 	on_session_quit();
 }
 
-void Gobby::Window::on_obby_chat(obby::user& user, const Glib::ustring& message)
-{
-	// Got chat message
-	m_chat.obby_message(user, message);
-}
-
-void Gobby::Window::on_obby_server_chat(const Glib::ustring& message)
-{
-	// Got server chat message
-	m_chat.obby_server_message(message);
-}
-
-void Gobby::Window::on_obby_user_join(obby::user& user)
+void Gobby::Window::on_obby_user_join(const obby::user& user)
 {
 	// Tell user join to components
-	m_header.obby_user_join(user);
 	m_folder.obby_user_join(user);
 	m_userlist.obby_user_join(user);
+	m_documentlist.obby_user_join(user);
 	m_chat.obby_user_join(user);
 	m_statusbar.obby_user_join(user);
 }
 
-void Gobby::Window::on_obby_user_part(obby::user& user)
+void Gobby::Window::on_obby_user_part(const obby::user& user)
 {
 	// Tell user part to components
-	m_header.obby_user_part(user);
 	m_folder.obby_user_part(user);
 	m_userlist.obby_user_part(user);
+	m_documentlist.obby_user_part(user);
 	m_chat.obby_user_part(user);
 	m_statusbar.obby_user_part(user);
 }
 
-void Gobby::Window::on_obby_user_colour(obby::user& user)
+void Gobby::Window::on_obby_user_colour(const obby::user& user)
 {
 	m_userlist.obby_user_colour(user);
+	m_documentlist.obby_user_colour(user);
 	m_folder.obby_user_colour(user);
 }
 
@@ -730,37 +952,11 @@ void Gobby::Window::on_obby_document_insert(obby::document_info& document)
 	obby::local_document_info& local_doc =
 		dynamic_cast<obby::local_document_info&>(document);
 
-	m_header.obby_document_insert(local_doc);
 	m_folder.obby_document_insert(local_doc);
 	m_userlist.obby_document_insert(local_doc);
+	m_documentlist.obby_document_insert(local_doc);
 	m_chat.obby_document_insert(local_doc);
 	m_statusbar.obby_document_insert(local_doc);
-
-	// Get last page (the newly inserted one)
-	DocWindow* doc = static_cast<DocWindow*>(
-		m_folder.get_nth_page(m_folder.get_n_pages() - 1) );
-
-	// Set the path from which this document was opened, if we opened that
-	// file.
-	if(document.get_owner() == &m_buffer->get_self() &&
-	   !m_local_file_path.empty() )
-	{
-		// Select newly created page
-		m_folder.set_current_page(m_folder.get_n_pages() - 1);
-		doc->get_document().grab_focus();
-
-		// " " is newly created, so we do not need a path
-		if(m_local_file_path != " ")
-			doc->get_document().set_path(m_local_file_path);
-
-		// Crear local path
-		m_local_file_path.clear();
-	}
-
-#if 0
-	doc->get_document().signal_drag_data_received().connect(
-		sigc::mem_fun(*this, &Window::on_drag_data_received) );
-#endif
 }
 
 void Gobby::Window::on_obby_document_remove(obby::document_info& document)
@@ -768,23 +964,22 @@ void Gobby::Window::on_obby_document_remove(obby::document_info& document)
 	obby::local_document_info& local_doc =
 		dynamic_cast<obby::local_document_info&>(document);
 
-	m_header.obby_document_remove(local_doc);
 	m_folder.obby_document_remove(local_doc);
 	m_userlist.obby_document_remove(local_doc);
+	m_documentlist.obby_document_remove(local_doc);
 	m_chat.obby_document_remove(local_doc);
 	m_statusbar.obby_document_remove(local_doc);
-
-	// Reset title if last document has been closed
-	if(m_buffer->document_count() == 1)
-		set_title("Gobby");
 }
 
-Gobby::Document& Gobby::Window::get_current_document()
+Gobby::Document* Gobby::Window::get_current_document()
 {
+	// No document open
+	if(m_folder.get_n_pages() == 0) return NULL;
+
 	// Get currently selected page
 	Widget* page = m_folder.get_nth_page(m_folder.get_current_page() );
 	// Convert to document
-	return static_cast<DocWindow*>(page)->get_document();
+	return &static_cast<DocWindow*>(page)->get_document();
 }
 
 void Gobby::Window::apply_preferences()
@@ -793,8 +988,17 @@ void Gobby::Window::apply_preferences()
 		m_preferences.appearance.toolbar_show);
 }
 
-void Gobby::Window::update_title_bar(const Document& document)
+void Gobby::Window::update_title_bar()
 {
+	// No document
+	if(m_folder.get_n_pages() == 0)
+	{
+		set_title("Gobby");
+		return;
+	}
+
+	// Get currently active document
+	const Document& document = *get_current_document();
 	// Get title of current document
 	const Glib::ustring& file = document.get_title();
 	// Get path of current document
@@ -851,8 +1055,9 @@ void Gobby::Window::open_local_file(const Glib::ustring& file)
 			convert_to_utf8(Glib::file_get_contents(file)) );
 		convert2unix(content);
 
-		m_buffer->create_document(
-			Glib::path_get_basename(file), content);
+		m_buffer->document_create(
+			Glib::path_get_basename(file), content
+		);
 	}
 	catch(Glib::Exception& e)
 	{
@@ -863,29 +1068,34 @@ void Gobby::Window::open_local_file(const Glib::ustring& file)
 
 void Gobby::Window::save_local_file(Document& doc, const Glib::ustring& file)
 {
-	// TODO: Replace the following by a call to obby?
-	// Build ofstream
+	// Open stream to file
 	std::ofstream stream(file.c_str() );
 
-	// Could it be opened?
-	if(stream)
+	try
 	{
+		if(!stream)
+		{
+			obby::format_string str(
+				_("Could not open file '%0%' for writing")
+			);
+			str << file;
+			throw std::runtime_error(str.str() );
+		}
+
 		// Save content into file
-		stream << doc.get_content().raw() << std::endl;
+		stream << doc.get_content().raw();
+		stream.close();
+
 		// Set path of document
 		doc.set_path(file);
 		// Update title bar according to new path
-		update_title_bar(doc);
+		update_title_bar();
 		// Unset modifified flag
 		doc.get_buffer()->set_modified(false);
 	}
-	else
+	catch(std::exception& e)
 	{
-		// File could not be opened
-		obby::format_string str(
-			"Could not open file '%0%' for writing");
-		str << file;
-		display_error(str.str() );
+		display_error(e.what() );
 	}
 }
 
@@ -894,56 +1104,69 @@ void Gobby::Window::close_document(DocWindow& document)
 	// Check for the document being modified
 	if(document.get_document().get_modified() )
 	{
-		// Setup confirmation string
-		obby::format_string str(_(
-			"The document \"%0%\" has been changed since it was "
-			"saved to disk. Are you sure that you want to close it?"
-		) );
-		str << document.get_document().get_title();
+		// Setup confirmation strings
+		obby::format_string primary_str(
+			_("Save changes to document \"%0%\" before closing?")
+		);
+
+		obby::format_string secondary_str(
+			_("If you don't save, changes will be discarded, but "
+			  "may still be retrieved if you re-subscribe to the "
+			  "document.")
+		);
+
+		primary_str << document.get_document().get_title();
 
 		// Setup dialog
-		Gtk::MessageDialog dlg(*this, str.str(), false,
-			Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_YES_NO, true);
+		Gtk::MessageDialog dlg(*this, primary_str.str(), false,
+			Gtk::MESSAGE_WARNING, Gtk::BUTTONS_NONE, true);
+
+		// Set secondary text
+		dlg.set_secondary_text(secondary_str.str() );
+
 		// Add button to allow the user to save the dialog
-		dlg.add_button(Gtk::Stock::SAVE, Gtk::RESPONSE_APPLY);
+		dlg.add_button("Close without saving", Gtk::RESPONSE_REJECT);
+		dlg.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
+		dlg.add_button(Gtk::Stock::SAVE, Gtk::RESPONSE_ACCEPT)->
+			grab_focus();
 
 		// Show the dialog
 		int result = dlg.run();
 
 		switch(result)
 		{
-		case Gtk::RESPONSE_NO:
-			/* Don't close it */
-			return;
+		case Gtk::RESPONSE_REJECT:
+			/* Close the document */
 			break;
-		case Gtk::RESPONSE_YES:
-			/* Yes, close it */
-			break;
-		case Gtk::RESPONSE_APPLY:
+		case Gtk::RESPONSE_ACCEPT:
 			/* Save the document before closing it */
 			m_folder.set_current_page(m_folder.page_num(document) );
+			// TODO: Do not close the document if the user cancells
+			// the save dialog.
 			on_document_save();
 			break;
+		case Gtk::RESPONSE_CANCEL:
+		case Gtk::RESPONSE_DELETE_EVENT:
+			/* Do not close the document */
+			return;
+			break;
+		default:
+			throw std::logic_error("Gobby::Window::close_document");
+			break;
 		}
 	}
 
 	if(m_buffer.get() != NULL)
 	{
-		// Send remove document request
-		m_buffer->remove_document(
-			document.get_document().get_document()
-		);
+		// Unsubscribe from document
+		document.get_document().get_document().unsubscribe();
 	}
 	else
 	{
 		// Buffer does not exist: Maybe the connection has been lost
 		// or something: Just remove the document from the folder.
 		m_folder.remove_page(document);
-
-		// If there are no more documents, disable
-		// save and close buttons in header
-		if(!m_folder.get_n_pages() )
-			m_header.disable_document_actions();
+		on_folder_document_remove(document);
 	}
 }
 
@@ -954,4 +1177,3 @@ void Gobby::Window::display_error(const Glib::ustring& message,
 	                       Gtk::BUTTONS_OK, true);
 	dlg.run();
 }
-
